// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/driver/ccur.c $
// $Rev: 51090 $
// $Date: 2022-05-31 10:48:15 -0500 (Tue, 31 May 2022) $

// 16AO64C: Device Driver: source file: Concurrent specific modifications

#include "main.h"

#ifdef CCUR_BUILD

// This code was developed by or based directly on code developed by Concurrent
// Computer Corporation while working around issues with the PEX8311, which is
// used on the PCIE-16AO64C.



//*****************************************************************************
/*** Fix Slow DMA problem
 *** 1) set the prefetch size limit at location 0x0068 to 2048
 *** 2) set the prefetch size at location 0x000c to 512
 ***/
#define	GSC16AOC_PCI_VENDOR					0x10b5		/* PLX Technologies VendorID */
#define	GSC16AOC_PEX8111_DEVICE				0x8111		/* CCURDSCC PCIe-to-PCI Bridge */
#define	MAIN_CONTROL_REGISTER_INDEX_OFFSET	0x84
#define	MAIN_CONTROL_REGISTER_DATA_OFFSET	0x88
#define	PCI_CONTROL_REGISTER_OFFSET			0x0C		/* 0x100C */
#define	PCI_DEVICE_CONTROL_REGISTER_OFFSET	0x68

#define	PREFETCH_SIZE_LIMIT_SHIFT			12			/* in PCI_DEVICE_CONTROL_REGISTER */
#define	PREFETCH_SIZE_LIMIT_MASK			7			/* in PCI_DEVICE_CONTROL_REGISTER */
#define	PREFETCH_SIZE_LIMIT_128				0
#define	PREFETCH_SIZE_LIMIT_256				1
#define	PREFETCH_SIZE_LIMIT_512				2
#define	PREFETCH_SIZE_LIMIT_1024			3
#define	PREFETCH_SIZE_LIMIT_2048			4
#define	PREFETCH_SIZE_LIMIT_4096			5

#define	PREFETCH_SIZE_SHIFT					27			/* in PCI_CONTROL_REGISTER */
#define	PREFETCH_SIZE_MASK					7			/* in PCI_CONTROL_REGISTER */
#define	PREFETCH_SIZE_DISABLE				0
#define	PREFETCH_SIZE_64					1
#define	PREFETCH_SIZE_128					2
#define	PREFETCH_SIZE_256					3
#define	PREFETCH_SIZE_512					4
#define	PREFETCH_SIZE_1024					5
#define	PREFETCH_SIZE_2048					6
#define	PREFETCH_SIZE_4096					7			/* NEED TO ALSO SET LIMIT COMP BIT IN PECS_TLPCFG0 */


//*****************************************************************************
static void
_gsc16ao64c_set_prefetch_size(dev_data_t* device)
{
	os_pci_t	*pex8111_pdev = device->pci->bus->self;
	unsigned int reg;
	unsigned short	device_id;
	unsigned short	vendor;

	reg	= os_reg_pci_rx_u32(device, 1, 0);
	device_id = (reg & 0xFFFF0000) >> 16;
	vendor = reg & 0xFFFF;
	if((device_id != GSC16AOC_PEX8111_DEVICE) ||
		(vendor != GSC16AOC_PCI_VENDOR)) {
		// printk("prefetch_size(): vendor=0x%04x device=0x%04x\n", vendor,device_id);
		return;
	}

	/***
	 *** Set PCI_DEVICE_CONTROL_REGISTER @ 0x0068 to 2048
	 ***/
	os_reg_pci_mx_u32(
		device,
		1,
		PCI_DEVICE_CONTROL_REGISTER_OFFSET,
		PREFETCH_SIZE_LIMIT_2048 << PREFETCH_SIZE_LIMIT_SHIFT,
		PREFETCH_SIZE_LIMIT_MASK << PREFETCH_SIZE_LIMIT_SHIFT);

	/***
	 *** Set PCI_CONTROL_REGISTER @ 0x100C to 2048
	 ***/
	/* First select the PCI_CONTROL_REGISTER_OFFSET register */
	os_reg_pci_tx_u32(	device,
						1,
						MAIN_CONTROL_REGISTER_INDEX_OFFSET,
						PCI_CONTROL_REGISTER_OFFSET);
	/* Now, change the PREFETCH SIZE TO A GOOD VALUE */
	/* It is observed that any size between 512 and 4096 result in the same
	 * performance. Larger values means that the device would lock the bus for
	 * a longer time. Hence, we use 2048 prefetch size.
	 */
	os_reg_pci_mx_u32(
		device,
		1,
		MAIN_CONTROL_REGISTER_DATA_OFFSET,
		PREFETCH_SIZE_2048 << PREFETCH_SIZE_SHIFT,
		PREFETCH_SIZE_MASK << PREFETCH_SIZE_SHIFT);
}



//*****************************************************************************
void ccur_dma_int_extern(dev_data_t* dev)
{
	/* CCUR FIX - stop spureous interrupts being generated */
	#define	BCR_IRQ_REQUEST (1 << 11)

	// Clear the device interrupt.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, 0, BCR_IRQ_REQUEST);
}



//*****************************************************************************
int	ccur_dev_device_create_extern(dev_data_t* dev)
{
	_gsc16ao64c_set_prefetch_size(dev);
	return(0);
}



//*****************************************************************************
long ccur_pio_write_available_extern(dev_data_t* dev, size_t count)
{
	u32		bsr;
	long	bytes;
	u32		samples;

	bsr		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

	/* Should never happen. HW fifo size is currently 256K samples */
	if((bsr & 0x7FFFF) > (dev->cache.fifo_size-1))
		return(0);	/* CCUR - safety sanity check */

	samples	= (dev->cache.fifo_size - 1) - (bsr & 0x3FFFF);
	count	= samples * 4;
	return(count);
}


#endif
