// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/driver/ccur.h $
// $Rev: 51373 $
// $Date: 2022-07-13 12:59:25 -0500 (Wed, 13 Jul 2022) $

// 16AO64C: Device Driver: header file: Concurrent specific modifications

#ifndef	__CCUR_H__
#define	__CCUR_H__

#include "16ao64c.h"

#ifdef CCUR_BUILD

// While chasing down the cause of DMA related problems with the PCIE-16AO64C,
// numerous, varied error conditions arose. While corrective measures were
// implemented in the driver by Concurrent for these error conditions, all of
// the error conditions were symptoms of the problem rather than the root
// cause. Consequently, the corrective measures addressed only symptoms. The
// root cause has been determined to be an unknown issue on the internal PCI
// bus of the PEX8311. (The specifics of the problem have not been identified.)
// A work around for this internal PEX8311 issue has been implemented in
// 16AO64C firmware and is available in firmware version 400 and later. This
// work around requires corresponding driver and/or application changes. The
// GSC driver sources have been modified to properly handle all firmware
// variations. It was noted that the host and expansion chassi seemed to be a
// factor in the occurance of the problems. With some hosts the problem did not
// appear, even after over 24 hours of continuous testing and no corrective
// measures enabled in the driver. With other host/chassis combinations the
// full measure of driver corrections were required.



// DMA Command Code ***********************************************************
// Changing of the DMA Command Code was moved to gsc_dma_create() in gsc_dma.c.
// CCUR Comments
	/* increase DMA burst size overall */
	/* a DMA of 8K samples (32kBytes) was taking 1.7ms on the
	 * OneStop chassis. With the fix, it went down to
	 * 0.3ms/dma. Improvements are also seen when devices are
	 * installed on the mother board.
	 */

// DMA Buffer Allocation ******************************************************
// This code has already been converted to use __get_free_pages().
// The same goes for the GFP_DMA32 flag.

// gsc_dma.o ******************************************************************
// The clearing of the firmware interrupt is retained and is accessigne via
// a macro. This change was not incorporated into GSC code as clearing of
// firmware interrupts is already handled by the ISR.

#define	GSC_DMA_CHANNEL_RESET_EXTERN(dev,dma)	ccur_clear_bcr_irq((dev))

// The conditional DMA Abort sequence was implemented in the GSC code to
// occur with each DMA channel reset.

// gsc_irq.c ******************************************************************
// The Abort interrupt is enabled in INTCSR by default and is left that way.
// Details of the interrupt are posted to the system log.
// Abort interrupts are serviced, but the Abort interrupt is not disabled.
// Your clearing of firmware interrupts during DMA0 and DMA1 interrupts is
// retained, but is access via macros. This change was not incorporated into
// GSC code as clearing of firmware interrupts is already handled by the ISR.

#define	GSC_DMA0_INT_EXTERN(dev)			ccur_clear_bcr_irq((dev))
#define	GSC_DMA1_INT_EXTERN(dev)			ccur_clear_bcr_irq((dev))

// gsc_write.c ****************************************************************
// The fix of changing "if (send > count) send = count;" to "if (send > want)
// send = want;", at line 109 of your file, has been incorporated.
// You made two mods changing "if (sent)" to "if (sent >= 0)". In both
// instances your comments were "case of sent == 0 was being ignored", which is
// not correct. In the first instance, at line 161 of your file version, the
// case of zero in the original code results in a break from the main loop
// (line 174). This ends the write, which is appropriate for a non-blocking
// write request. Your modification causes the write call to enter a polling
// loop, which is not appropriate for a non-blocking call when the FIFO is
// full. In addition, the tight polling significent increases CPU loading. This
// change was not carried forward. In the second instance, at line 185 of your
// file version, the case of zero in the original code results in a sleep of
// upto one system timer period. The purpose of the sleep is to allow time for
// more data to exit the FIFO. Your modification causes the write call to enter
// a tight polling mode, which significent increases CPU loading. This is
// typically undesirable. However, to accommodate your option, Ive added a
// macro that allows a CCUR build to operate in the tight polling mode. This
// change was not incorporated into GSC code due to the high CPU loading.

#define	GSC_WRITE_ENABLE_TIGHT_POLLING

// write.c ********************************************************************
// Your PIO Write Available mod has been implemented as a macro. Your mod of
// subtracting one in the computation was not carried forward in the GSC code
// as a correction was made by changing the mask from 0x3FFFF to 0x7FFFF.

#define	PIO_WRITE_AVAILABLE_EXTERN(dev,cnt)		ccur_pio_write_available_extern((dev),(cnt))

// Your DMDMA Write Available mod has been implemented as a macro. This
// change was not incorporated into GSC code due to its impact on performance.
// CCUR comments:
	/* For now, we need to throttle Demand DMA so that it never writes
	 * to exceed the FIFO size as there appears to be a problem with the
	 * hardware where it causes the device to generate numerous DMA aborts
	 * on certain devices and at times corrupts the PEX8111 data on it.
	 * Result is that the device could stop interrupting and also cause
	 * other devices to do the same.
	 */

#define	DMDMA_WRITE_AVAILABLE_EXTERN(dev,cnt)	ccur_pio_write_available_extern((dev),(cnt))

// The debug code that checks for corrupted data has not been retained.

// device.c *******************************************************************
// The code you added to change some of the PEX8311 settings was been retained
// and is available via a macro. This change was not incorporated into GSC code
// as the defaults provided in the device's EEPROM provide better performance
// for most hosts.

#define	DEV_DEVICE_CREATE_EXTERN(dev)			ccur_dev_device_create_extern((dev))

// aout ***********************************************************************
// Command line options have been expanded. The -ch# option specifies that the
// output be generated for # number of channels (1-64). This is automatically
// reduced accordingly if the device doesn't have the specified number of
// channels. The -P# option specifies the output patterns to generate; -PA =
// all, -P1 = saw tooth, -P2 = ramp, -P3 = square wave, -P4 = square wave. The
// -s option, if given, causes the output data to be saved to aout.txt before
// it is written to the device.

// txrate *********************************************************************
// This application was modified to have three selectable output patterns.
// 1. square wave, 64 samples at 0x0000, 64 samples at 0xFFFF
// 2. square wave, 4 samples at 0x000F, 4 samples at 0xFFF0
// 3. square wave, 1 sample at 0 volts, 1 sample at channel specific voltage



// prototypes *****************************************************************

void	ccur_clear_bcr_irq(dev_data_t* dev);
int		ccur_dev_device_create_extern(dev_data_t* dev);
long	ccur_pio_write_available_extern(dev_data_t* dev, size_t count);



#endif
#endif
