// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/driver/ioctl.c $
// $Rev: 53038 $
// $Date: 2023-05-18 15:57:30 -0500 (Thu, 18 May 2023) $

// 16AO64C: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTOCAL_START			D13		// BCR
#define	AUTOCAL_PASS			D14		// BCR
#define	INIT_START				D15		// BCR

#define	IRQ_AUTOCAL_DONE		0x0100	// BCR
#define	IRQ_MASK				0x0700	// BCR
#define	IRQ_ACTIVE				0x0800	// BCR



//*****************************************************************************
static void _bcr_mod(dev_data_t* dev, u32 value, u32 mask)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);
}



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	int		ret	= 0;

	switch (arg[0])
	{
		default:	arg[0]	= AO64C_IOCTL_QUERY_ERROR;
					ret		= -EINVAL;
					break;

		case AO64C_QUERY_AUTOCAL_MS:	arg[0]	= dev->cache.autocal_ms;	break;
		case AO64C_QUERY_CHANNEL_MAX:	arg[0]	= dev->cache.channels_max;	break;
		case AO64C_QUERY_CHANNEL_QTY:	arg[0]	= dev->cache.channel_qty;	break;
		case AO64C_QUERY_COUNT:			arg[0]	= AO64C_QUERY_LAST;			break;
		case AO64C_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;			break;
		case AO64C_QUERY_DIFFERENTIAL:	arg[0]	= dev->cache.differential;	break;
		case AO64C_QUERY_DISCONNECT:	arg[0]	= dev->cache.disconnect;	break;
		case AO64C_QUERY_FIFO_SIZE:		arg[0]	= dev->cache.fifo_size;		break;
		case AO64C_QUERY_FILTER:		arg[0]	= dev->cache.filter;		break;
		case AO64C_QUERY_FREF_DEFAULT:	arg[0]	= dev->cache.fref_default;	break;
		case AO64C_QUERY_FSAMP_MAX:		arg[0]	= dev->cache.fsamp_max;		break;
		case AO64C_QUERY_FSAMP_MIN:		arg[0]	= dev->cache.fsamp_min;		break;
		case AO64C_QUERY_INIT_MS:		arg[0]	= dev->cache.initialize_ms;	break;
		case AO64C_QUERY_LAST:			arg[0]	= AO64C_QUERY_LAST;			break;
		case AO64C_QUERY_NRATE_MASK:	arg[0]	= dev->cache.nrate_mask;	break;
		case AO64C_QUERY_NRATE_MAX:		arg[0]	= dev->cache.nrate_max;		break;
		case AO64C_QUERY_NRATE_MIN:		arg[0]	= dev->cache.nrate_min;		break;
		case AO64C_QUERY_RANGE:			arg[0]	= dev->cache.range;			break;
	}

	return(ret);
}



//*****************************************************************************
static int _init_start(dev_data_t* dev, void* arg)
{
	// Initiate initialization.
	os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, INIT_START);
	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	int			i;
	int			mask;
	long		ms			= dev->cache.initialize_ms + 5000;
	long		ms_total	= ms;
	u32			reg;
	int			ret			= 0;
	os_sem_t	sem;
	int			tmp;
	VADDR_T		va			= dev->vaddr.gsc_bcr_32;
	int			value;
	gsc_wait_t	wt;

	if ((dev->irq.opened) && (gsc_global.driver_unloading == 0))
	{
		ms_total	*= 2;

		// Safely select the Initialize Done interrupt.
		mask	= IRQ_ACTIVE | IRQ_MASK;
		value	= 0;
		os_reg_mem_mx_u32(dev, va, value, mask);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= AO64C_WAIT_GSC_INIT_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _init_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: INITILIZE DONE IRQ TIMED OUT AFTER %ld ms.\n",
					dev->model,
					(long) ms);
		}
	}
	else
	{
		_init_start(dev, NULL);
	}

	// Manually wait for completion as the IRQ wait may have ended early.
	tmp	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_start(dev_data_t* dev, void* arg)
{
	// Initiate autocalibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTOCAL_START, AUTOCAL_START);
	return(0);
}



//*****************************************************************************
static int _autocal(dev_data_t* dev, void* arg)
{
	u32			mask;
	long		ms		= dev->cache.autocal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	VADDR_T		va		= dev->vaddr.gsc_bcr_32;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Autocal Done interrupt.
	mask	= IRQ_MASK | IRQ_ACTIVE;
	value	= IRQ_AUTOCAL_DONE;
	_bcr_mod(dev, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AO64C_WAIT_GSC_AUTOCAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _autocal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE DONE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, va, AUTOCAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & AUTOCAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, va);

	if ((reg & AUTOCAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _autocal_status(dev_data_t* dev, s32* arg)
{
	u32	reg;
	s32	value;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & AUTOCAL_START)
		value	= AO64C_AUTOCAL_STATUS_ACTIVE;
	else if (reg & AUTOCAL_PASS)
		value	= AO64C_AUTOCAL_STATUS_PASS;
	else
		value	= AO64C_AUTOCAL_STATUS_FAIL;

	arg[0]	= value;
	return(0);
}



//*****************************************************************************
static int _buffer_clear(dev_data_t* dev, void* arg)
{
	#define	CLEAR	D11
	#define	DATA	D16
	#define	FRAME	D17

	int		i;
	u32		reg;
	int		ret	= 0;
	VADDR_T	va	= dev->vaddr.gsc_bor_32;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, va, CLEAR, CLEAR);

	// Wait for the bit to clear.

	for (i = 0;; i++)
	{
		reg	= os_reg_mem_rx_u32(dev, va);

		if ((reg & CLEAR) == 0)
			break;

		if (i >= 250)
		{
			ret	= -EINVAL;
			printf(	"%s: The analog output buffer took too long to clear.\n",
					DEV_NAME);
			break;
		}
	}

	// Clear the Data Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, DATA);

	// Clear the Frame Overflow status bit.
	os_reg_mem_mx_u32(dev, va, 0, FRAME);

	return(ret);
}



//*****************************************************************************
static int _buffer_level(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_bsr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _buffer_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BUFFER_MODE_OPEN,
		AO64C_BUFFER_MODE_CIRC,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _buffer_over_data(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BUFFER_OVER_DATA_NO,
		AO64C_BUFFER_OVER_DATA_YES,
		-1	// terminate list
	};

	int	ret;

	if ((arg[0] == AO64C_BUFFER_OVER_DATA_CLR) ||
		(arg[0] == AO64C_BUFFER_OVER_DATA_CHK))
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 16, 16);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _buffer_over_frame(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BUFFER_OVER_FRAME_NO,
		AO64C_BUFFER_OVER_FRAME_YES,
		-1	// terminate list
	};

	int	ret;

	if ((arg[0] == AO64C_BUFFER_OVER_FRAME_CLR) ||
		(arg[0] == AO64C_BUFFER_OVER_FRAME_CHK))
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 17, 17);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _buffer_status(dev_data_t* dev, s32* arg)
{
	u32	bor;
	u32	btr;

	bor	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bor_32);

	if (bor & D15)
	{
		arg[0]	= AO64C_BUFFER_STATUS_FULL;
	}
	else if (bor & D12)
	{
		arg[0]	= AO64C_BUFFER_STATUS_EMPTY;
	}
	else
	{
		btr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_btr_32);

		if (btr & D20)
			arg[0]	= AO64C_BUFFER_STATUS_HIGH;
		else
			arg[0]	= AO64C_BUFFER_STATUS_LOW;
	}

	return(0);
}



//*****************************************************************************
static int _buffer_thresh_lvl(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_btr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _buffer_thresh_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BUFFER_THRESH_STS_CLR,
		AO64C_BUFFER_THRESH_STS_SET,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_btr_32, 20, 20);
	return(ret);
}



//*****************************************************************************
static int _burst_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BURST_ENABLE_NO,
		AO64C_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 2, 2);
	return(ret);
}



//*****************************************************************************
static int _burst_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BURST_READY_NO,
		AO64C_BURST_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 3, 3);
	return(ret);
}



//*****************************************************************************
static int _burst_trigger(dev_data_t* dev, s32* arg)
{
	int	ret;

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, D4, D4);
	ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_bor_32, D4, 0);
	return(ret);
}



//*****************************************************************************
static int _burst_trig_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_BURST_TRIG_SRC_EXT,
		AO64C_BURST_TRIG_SRC_RBG,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 1, 1);
	return(ret);
}



//*****************************************************************************
static int _clock_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_CLOCK_ENABLE_NO,
		AO64C_CLOCK_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _clock_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_CLOCK_MODE_TARGET,
		AO64C_CLOCK_MODE_INITIATOR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 0, 0);
	return(ret);
}



//*****************************************************************************
static int _clock_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_CLOCK_READY_NO,
		AO64C_CLOCK_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _clock_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_CLOCK_SRC_RAG,
		AO64C_CLOCK_SRC_EXT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 0, 0);
	return(ret);
}



//*****************************************************************************
static int _clock_sw(dev_data_t* dev, s32* arg)
{
	int	ret;

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, D7, D7);
	ret	= gsc_poll_u32(dev, dev->io.tx.timeout_s * 1000, dev->vaddr.gsc_bor_32, D7, 0);
	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_DATA_FORMAT_2S_COMP,
		AO64C_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _ground_sense(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_GROUND_SENSE_DISABLE,
		AO64C_GROUND_SENSE_REMOTE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 3, 3);
	return(ret);
}



//*****************************************************************************
static int _irq_sel(dev_data_t* dev, s32* arg)
{
	u32	reg;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		reg		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
		arg[0]	= GSC_FIELD_DECODE(reg, 10, 8);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case AO64C_IRQ_INIT_DONE:
			case AO64C_IRQ_AUTOCAL_DONE:
			case AO64C_IRQ_BUF_EMPTY:
			case AO64C_IRQ_BUF_THRESH_LO2HI:
			case AO64C_IRQ_BUF_THRESH_HI2LO:
			case AO64C_IRQ_BURST_TRIG_READY:
			case AO64C_IRQ_LOAD_READY:
			case AO64C_IRQ_LOAD_READY_END:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, arg[0] << 8, 0xF00);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _load_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_LOAD_READY_NO,
		AO64C_LOAD_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bor_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _load_request_start(dev_data_t* dev, void* arg)
{
	#define	BOR_REQUEST	D9

	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bor_32, BOR_REQUEST, BOR_REQUEST);
	return(0);
}



//*****************************************************************************
static int _load_request(dev_data_t* dev, s32* arg)
{
	#define	BCR_IRQ_LOAD_READY	0x0600
	#define	BOR_READY			D10

	long		ms;
	int			ret;
	int			ret2;
	os_sem_t	sem;
	gsc_wait_t	wt;

	// Safely select the Autocal Done interrupt.
	_bcr_mod(dev, BCR_IRQ_LOAD_READY, IRQ_MASK);
	_bcr_mod(dev, 0, IRQ_ACTIVE);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	ms				= dev->io.tx.timeout_s * 1000;
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AO64C_WAIT_GSC_LOAD_READY;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _load_request_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_DONE)
	{
	}
	else if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
	}
	else
	{
		// Manually wait for completion.
		ret2	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bor_32, BOR_READY, BOR_READY);
		ret		= ret ? ret : ret2;
	}

	return(ret);
}



//*****************************************************************************
static int _output_disconnect(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_OUTPUT_DISCONNECT_NO,
		AO64C_OUTPUT_DISCONNECT_YES,
		-1	// terminate list
	};

	int	ret;

	if (dev->cache.disconnect)
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 2, 2);
	}
	else
	{
		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case -1:

				arg[0]	= AO64C_OUTPUT_DISCONNECT_NO;
				ret		= 0;
				break;

			case AO64C_OUTPUT_DISCONNECT_NO:

				ret		= 0;
				break;

			case AO64C_OUTPUT_DISCONNECT_YES:

				arg[0]	= -1;
				ret		= 0;
				break;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _rag_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_RAG_ENABLE_NO,
		AO64C_RAG_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _rag_nrate(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.nrate_min,
			dev->cache.nrate_max,
			dev->vaddr.gsc_ragr_32,
			23, 0);
	return(ret);
}



//*****************************************************************************
static int _range(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_RANGE_10V,
		AO64C_RANGE_5V,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _rbg_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_RBG_ENABLE_NO,
		AO64C_RBG_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _rbg_nrate(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			dev,
			arg,
			dev->cache.nrate_min,
			dev->cache.nrate_max,
			dev->vaddr.gsc_rbgr_32,
			23, 0);
	return(ret);
}



//*****************************************************************************
static int _trigger_mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_TRIGGER_MODE_TARGET,
		AO64C_TRIGGER_MODE_INITIATOR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 1, 1);
	return(ret);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev, &dev->io.tx);
	return(0);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_over_data(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_TX_IO_OVER_DATA_IGNORE,
		AO64C_TX_IO_OVER_DATA_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.overflow_data);
	return(ret);
}



//*****************************************************************************
static int _tx_io_over_frame(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		AO64C_TX_IO_OVER_FRAME_IGNORE,
		AO64C_TX_IO_OVER_FRAME_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.tx.overflow_frame);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AO64C_IO_TIMEOUT_MIN,
			AO64C_IO_TIMEOUT_INFINITE,
			&dev->io.tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _tx_io_byte_swap(dev_data_t* dev, s32* arg)
{
	int	ret;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			arg[0]	= dev->io.tx.io_byte_swap;
			ret		= 0;
			break;

		case AO64C_TX_IO_BYTE_SWAP_PERFORM:
		case AO64C_TX_IO_BYTE_SWAP_CHECK:
		case AO64C_TX_IO_BYTE_SWAP_DISABLE:

			dev->io.tx.io_byte_swap	= arg[0];
			ret						= 0;
			break;
	}

	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AO64C_IOCTL_REG_READ,				(void*) gsc_reg_read_ioctl		},
	{ AO64C_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ AO64C_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ AO64C_IOCTL_QUERY,				(void*) _query					},
	{ AO64C_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ AO64C_IOCTL_AUTOCAL,				(void*) _autocal				},
	{ AO64C_IOCTL_AUTOCAL_STATUS,		(void*) _autocal_status			},
	{ AO64C_IOCTL_BUFFER_CLEAR,			(void*) _buffer_clear			},
	{ AO64C_IOCTL_BUFFER_LEVEL,			(void*) _buffer_level			},
	{ AO64C_IOCTL_BUFFER_MODE,			(void*) _buffer_mode			},
	{ AO64C_IOCTL_BUFFER_OVER_DATA,		(void*) _buffer_over_data		},
	{ AO64C_IOCTL_BUFFER_OVER_FRAME,	(void*) _buffer_over_frame		},
	{ AO64C_IOCTL_BUFFER_STATUS,		(void*) _buffer_status			},
	{ AO64C_IOCTL_BUFFER_THRESH_LVL,	(void*) _buffer_thresh_lvl		},
	{ AO64C_IOCTL_BUFFER_THRESH_STS,	(void*) _buffer_thresh_sts		},
	{ AO64C_IOCTL_BURST_ENABLE,			(void*) _burst_enable			},
	{ AO64C_IOCTL_BURST_READY,			(void*) _burst_ready			},
	{ AO64C_IOCTL_BURST_TRIGGER,		(void*) _burst_trigger			},
	{ AO64C_IOCTL_BURST_TRIG_SRC,		(void*) _burst_trig_src			},
	{ AO64C_IOCTL_CLOCK_ENABLE,			(void*) _clock_enable			},
	{ AO64C_IOCTL_CLOCK_MODE,			(void*) _clock_mode				},
	{ AO64C_IOCTL_CLOCK_READY,			(void*) _clock_ready			},
	{ AO64C_IOCTL_CLOCK_SRC,			(void*) _clock_src				},
	{ AO64C_IOCTL_CLOCK_SW,				(void*) _clock_sw				},
	{ AO64C_IOCTL_DATA_FORMAT,			(void*) _data_format			},
	{ AO64C_IOCTL_GROUND_SENSE,			(void*) _ground_sense			},
	{ AO64C_IOCTL_IRQ_SEL,				(void*) _irq_sel				},
	{ AO64C_IOCTL_LOAD_READY,			(void*) _load_ready				},
	{ AO64C_IOCTL_LOAD_REQUEST,			(void*) _load_request			},
	{ AO64C_IOCTL_OUTPUT_DISCONNECT,	(void*) _output_disconnect		},
	{ AO64C_IOCTL_RAG_ENABLE,			(void*) _rag_enable				},
	{ AO64C_IOCTL_RAG_NRATE,			(void*) _rag_nrate				},
	{ AO64C_IOCTL_RANGE,				(void*) _range					},
	{ AO64C_IOCTL_RBG_ENABLE,			(void*) _rbg_enable				},
	{ AO64C_IOCTL_RBG_NRATE,			(void*) _rbg_nrate				},
	{ AO64C_IOCTL_TRIGGER_MODE,			(void*) _trigger_mode			},
	{ AO64C_IOCTL_TX_IO_ABORT,			(void*) _tx_io_abort			},
	{ AO64C_IOCTL_TX_IO_MODE,			(void*) _tx_io_mode				},
	{ AO64C_IOCTL_TX_IO_OVER_DATA,		(void*) _tx_io_over_data		},
	{ AO64C_IOCTL_TX_IO_OVER_FRAME,		(void*) _tx_io_over_frame		},
	{ AO64C_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ AO64C_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ AO64C_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ AO64C_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},
	{ AO64C_IOCTL_TX_IO_BYTE_SWAP,		(void*) _tx_io_byte_swap		},
	{ -1, NULL }
};


