// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/driver/irq.c $
// $Rev: 53038 $
// $Date: 2023-05-18 15:57:30 -0500 (Thu, 18 May 2023) $

// 16AO64C: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	#define	BCR_IRQ_REQUEST	D11

	u32	bcr;
	u32	irq;

	bcr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_bcr_32);

	if (bcr & BCR_IRQ_REQUEST)
	{
		// Clear the interrupt.
		bcr	^= BCR_IRQ_REQUEST;
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, bcr);

		// Resume any waiting threads.
		irq	= GSC_FIELD_DECODE(bcr, 10, 8);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AO64C_IRQ_INIT_DONE:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_INIT_DONE);
				break;

			case AO64C_IRQ_AUTOCAL_DONE:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_AUTOCAL_DONE);
				break;

			case AO64C_IRQ_BUF_EMPTY:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_BUF_EMPTY);
				break;

			case AO64C_IRQ_BUF_THRESH_LO2HI:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_BUF_THRESH_LO2HI);
				break;

			case AO64C_IRQ_BUF_THRESH_HI2LO:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_BUF_THRESH_HI2LO);
				break;

			case AO64C_IRQ_BURST_TRIG_READY:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_BURST_TRIG_READY);
				break;

			case AO64C_IRQ_LOAD_READY:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_LOAD_READY);
				break;

			case AO64C_IRQ_LOAD_READY_END:

				gsc_wait_resume_irq_gsc(dev, AO64C_WAIT_GSC_LOAD_READY_END);
				break;
		}
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_bcr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, 0);

	gsc_irq_destroy(dev);
}


