// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/samples/dcout/perform.c $
// $Rev: 51077 $
// $Date: 2022-05-31 10:30:29 -0500 (Tue, 31 May 2022) $

// 16AO64C: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _adjust_output(const args_t* args, int errs)
{
	char	buf[1024];
	char	c;
	u32		data[1];	// digital value for one channel
	u32		dig;
	int		i;
	int		len;
	int		range	= (args->range == AO64C_RANGE_10V) ? 10 : 5;
	int		ret;
	float	value;

	gsc_label("Output Adjustments");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("ABORTING  (errors reported)\n");
			break;
		}

		// Prompt for input then update the output voltage.
		printf("\n");

		for (;;)
		{
			printf(	"\nEnter a value from +%d.0 to -%d.0,"
					" or enter 'exit' or 'x' to quit.\n",
					range, range);
			fgets(buf, sizeof(buf), stdin);

			// Clear the EOL characters.
			len	= (int) strlen(buf);

			for (i = 0; i < 2; i++)
			{
				if ((len >= 1) && ((buf[len - 1] == '\n') || (buf[len - 1] == '\r')))
				{
					len--;
					buf[len]	= 0;
				}
			}

			if (buf[0] == 0)
			{
				// No input was provided. Repost the prompt.
				continue;
			}

			// Check for an EXIT request.
			if ((strcmp(buf, "exit") == 0) || (strcmp(buf, "x") == 0))
			{
				printf("... exiting\n\n");
				break;
			}

			// Convert the input to a floating point value.
			i	= sscanf(buf, "%f%c", &value, &c);

			if (i != 1)
			{
				// Extraneous input was provided at the prompt.
				printf(" ... invalid input\n");
				continue;
			}

			// Convert the value from floating point to 16-bit Offset Binary.

			if (value >= ((float) +range))
			{
				dig	= 0xFFFF;
			}
			else if (value <= ((float) -range))
			{
				dig	= 0x0000;
			}
			else
			{
				dig	= (u32) ((u16) ((value + range) * 0xFFFF / (2.0 * range)));
			}

			// Add the channel tags.

			for (i = 0; i < (int) SIZEOF_ARRAY(data); i++)
				data[i]	= dig | ((unsigned) i << 24);

			// Add the End Of Frame flag.
			data[SIZEOF_ARRAY(data) - 1]	|= 0x40000000;

			// Send the data.
			ret	= ao64c_write(args->fd, data, sizeof(data));

			if (ret == (int) sizeof(data))
			{
				printf(	"PASS: wrote data for %d channel%s.\n",
						(int) SIZEOF_ARRAY(data),
						(SIZEOF_ARRAY(data) == 1) ? "" : "s");
			}
			else
			{
				printf(	"FAILURE: ao64c_write() returned %d rather than %d.\n",
						ret,
						(int) sizeof(dig));
				errs++;
			}
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Analog Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ao64c_config_ao	(args->fd, -1, 1, 100000, 0x1);
	errs	+= ao64c_tx_io_mode	(args->fd, -1, 1, GSC_IO_MODE_PIO, NULL);
	errs	+= ao64c_range		(args->fd, -1, 1, args->range, NULL);
	errs	+= _adjust_output(args, errs);

	gsc_label_level_dec();
	return(errs);
}


