// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/samples/mbsync/main.c $
// $Rev: 53051 $
// $Date: 2023-05-19 11:03:01 -0500 (Fri, 19 May 2023) $

// 16AO64C: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg				values			desc
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",			{ -1 },			"Continue testing until an error occurs."				},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",			{ +1 },			"Continue testing even if errors occur."				},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",			{ 1 },			"Continue testing for up to # minutes."					},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",			{ 1 },			"Continue testing for up to # iterations."				},
		{	GSC_ARG_S32_RANGE,	&args->seconds,			"-s",			{ 1, 3600 },	"Outut data for # seconds."								},
		{	GSC_ARG_S32_FLAG,	&args->unsync,			"-U",			{ 1 },			"Configure the devices for unsychchronized operation."	},
		{	GSC_ARG_DEV_INDEX,	&args->index[0],		"initiator",	{ 9 },			"The zero based index of the initiator."				},
		{	GSC_ARG_DEV_INDEX,	&args->index[1],		"targets",		{ 9 },			"The zero based index of the one or more targets."		},
		{	GSC_ARG_DEV_INDEX,	&args->index[2],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[3],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[4],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[5],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[6],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[7],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[8],		NULL,			{ 9 },			NULL	},
		{	GSC_ARG_DEV_INDEX,	&args->index[9],		NULL,			{ 9 },			NULL	}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"mbsync",
		/* desc	*/	"Multi-Board Synchronization",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;
	int	i;
	int	j;

	for (i = 0; i < SIZEOF_ARRAY(args->index); i++)
		args->index[i]	= -1;

	memset(args, 0, sizeof(args_t));
	args->seconds	= 15;

	for (i = 0; i < SIZEOF_ARRAY(args->index); i++)
		args->index[i]	= -1;

	errs	= gsc_args_parse(argc, argv, &set);

	if ((errs == 0) && (args->index[0] < 0))
	{
		errs++;
		printf("  FAIL <---  (Initiator not specified.)\n");
	}

	if ((errs == 0) && (args->index[1] < 0))
	{
		errs++;
		printf("  FAIL <---  (No targets specified.)\n");
	}

	if ((errs == 0) && (args->index[SIZEOF_ARRAY(args->index) - 1] >= 0))
	{
		errs++;
		printf(	"  FAIL <---  (Too many devices specified.)\n"
				"  FAIL <---  (The limit is %d devices.)\n",
				(int) SIZEOF_ARRAY(args->index) - 1);
	}

	for (i = 0; (errs == 0) && (i < BOARD_QTY_MAX); i++)
	{
		if (args->index[i] < 0)
			continue;

		for (j = i + 1; j < BOARD_QTY_MAX; j++)
		{
			if (args->index[j] == args->index[i])
			{
				errs++;
				printf(	"  FAIL <---  (Device %d given more than once.)\n",
						(int) args->index[j]);
				break;
			}
		}
	}

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	const char*	comma	= "";
	int			i;

	gsc_label("Accessing Devices");

	for (i = 0; args->index[i] >= 0; i++)
	{
		printf("%s%d", comma, (int) args->index[i]);
		comma	= ", ";
	}

	printf("\n");
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(32);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= ao64c_init_util(1);
		errs	+= os_id_driver(ao64c_open, ao64c_read, ao64c_close);
		errs	+= ao64c_count_boards(1, &args.qty);

		if (args.qty < 2)
		{
			errs++;
			printf(	"  FAIL <---  (Too few devices detected.)\n");
		}

		// Open access to devices

		if (errs == 0)
			errs	= service_open(&args, args.fd);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= service_id_device(&args);

			// Perform testing

			errs	+= perform_tests(&args);
		}

		// Close device access

		errs	+= service_close(&args);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


