// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/samples/mbsync/service.c $
// $Rev: 53044 $
// $Date: 2023-05-18 16:00:46 -0500 (Thu, 18 May 2023) $

// 16AO64C: Sample Application: source file

#include "main.h"



//*****************************************************************************
int service_close(const args_t* args)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		i;

	gsc_label("Closing Devices");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Close #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);
		err		= ao64c_close_util(args->fd[i], -1, 0);
		errs	+= err;
		printf("%s\n", err ? "FAIL <---" : "PASS");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int service_config_ao(const args_t* args, const s32* chans, s32 fsamp)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		i;
	u64		map;

	gsc_label("Configure Devices");
	gsc_label_long_comma(fsamp);
	printf(" S/S\n");
	gsc_label_level_inc();

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Configure AO #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);

		if (chans[i] == 64)
			map	= (u64) 0xFFFFFFFFFFFFFFFFLL;
		else
			map	= (u64) 0x00000000FFFFFFFFLL;

		err		= ao64c_config_ao(args->fd[i], -1, 0, fsamp, map);
		errs	+= err;
		printf("%s\n", err ? "FAIL <---" : "PASS");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int service_id_device(const args_t* args)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		i;

	gsc_label("Identify Devices");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Device #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);
		printf("\n");
		err		= ao64c_id_device(args->fd[i], -1, 0);
		errs	+= err;
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int service_open(const args_t* args, int* fd)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		i;

	gsc_label("Opening Devices");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; i < BOARD_QTY_MAX; i++)
		fd[i]	= -1;

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Open #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);
		err		= ao64c_open_util(args->index[i], 0, -1, 0, &fd[i]);
		errs	+= err;
		printf("%s\n", err ? "FAIL <---" : "PASS");
	}

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int service_query_channels(const args_t* args, s32* chans)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	int		i;
	int		total	= 0;

	gsc_label("Device Channels");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Output Channels #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);
		chans[i]	= 64;
		err			= ao64c_query(args->fd[i], -1, 0, AO64C_QUERY_CHANNEL_QTY, &chans[i]);

		if (err)
		{
			errs++;
			printf("FAIL <---\n");
		}
		else
		{
			total	+= chans[i];
			printf("%d\n", (int) chans[i]);
		}
	}

	gsc_label("Total Channels");
	printf("%d\n", (int) total);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int service_sample_rate(const args_t* args, const s32* chans)
{
	char	buf[64];
	int		err;
	int		errs	= 0;
	s32		fsamp;
	int		i;
	u64		map;
	s32		total	= 0;

	gsc_label("Device Sample Rates");
	printf("\n");
	gsc_label_level_inc();

	for (i = 0; (i < BOARD_QTY_MAX) && (args->index[i] >= 0); i++)
	{
		sprintf(buf, "Sample Rate #%d, ", args->index[i]);

		if (i == 0)
			strcat(buf, "Initiator");
		else
			sprintf(buf + strlen(buf), "Target #%d", i);

		gsc_label(buf);

		if (chans[i] == 64)
			map	= (u64) 0xFFFFFFFFFFFFFFFFLL;
		else
			map	= (u64) 0x00000000FFFFFFFFLL;

		err	= ao64c_fsamp_ao_report_all(args->fd[i], -1, 0, map, &fsamp);

		if (err)
		{
			errs++;
			printf("FAIL <---\n");
		}
		else
		{
			total	+= fsamp;
			gsc_label_long_comma(fsamp);
			printf(" S/S\n");
		}
	}

	gsc_label("Overall Rate");
	gsc_label_long_comma(total);
	printf(" S/S\n");

	gsc_label_level_dec();
	return(errs);
}


