// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/samples/txrate/perform.c $
// $Rev: 53052 $
// $Date: 2023-05-19 11:03:55 -0500 (Fri, 19 May 2023) $

// 16AO64C: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M					(1024L * 1024L)



// variables ******************************************************************

static	u32		_tx[_1M];
static	long	_tx_size;



//*****************************************************************************
static int _pattern_square(s32 chans, int msb, int width, u32 low, u32 high)
{
	int		chan;
	u32*	dest;
	u32		gef		= msb ? 0x40000000L : 0x00400000L;	// Group End Flag
	int		i;
	int		length	= 2 * width;
	int		pat;
	int		pat_len;
	int		pat_qty;
	int		shift	= msb ? 24 : 16;
	u32		v;
	u32		vals[2]	= { low, high };

	// This pattern produces a square wave.
	// "width" samples are at the "low" value.
	// "width" samples are at the "high" value.
	// All channels produce identical output.
	memset(_tx, 0, sizeof(_tx));
	pat_len		= chans * length;
	pat_qty		= SIZEOF_ARRAY(_tx) / pat_len;
	_tx_size	= pat_qty * pat_len * 4;
	dest		= _tx;

	for (pat = 0; pat < pat_qty; pat++)
	{
		for (i = 0; i < length; i++)
		{
			v	= vals[(i / width) & 0x1];

			for (chan = 0; chan < chans; chan++)
			{
				dest[0]	= v;				// Data Value
				dest[0]	|= chan << shift;	// Channel Tag
				dest++;
			}

			dest[-1]	|= gef;				// Group End Flag
		}
	}

	return(0);
}



//*****************************************************************************
static int _pattern_width(const args_t* args, s32 chans, int msb)
{
	int	errs;
	int	width	= args->width / 2;

	// This produces a square wave X samples wide.
	// The first half at the minimum voltage.
	// The second half at the maximum voltage.
	// All channels produce identical output.
	errs	= _pattern_square(chans, msb, width, 0x0000, 0xFFFF);
	printf("Done  (%d sample square wave, at limits, all channels)\n", width * 2);
	return(errs);
}



//*****************************************************************************
static int _pattern_2(s32 chans, int msb)
{
	int		chan;
	u32*	dest;
	u32		gef		= msb ? 0x40000000L : 0x00400000L;	// Group End Flag
	int		pat;
	int		pat_len;
	int		pat_qty;
	int		shift	= msb ? 24 : 16;

	// Initialize the data with a square wave.
	// The square wave is two samples wide.
	// The first sample is non-zero with a channel specific voltage.
	// The second sample is zero.
	memset(_tx, 0, sizeof(_tx));
	pat_len		= chans * 2; // 1 at zero, 1 not at zero
	pat_qty		= SIZEOF_ARRAY(_tx) / pat_len;
	_tx_size	= pat_qty * pat_len * 4;
	dest		= _tx;

	for (pat = 0; pat < pat_qty; pat++)
	{
		// Channel 0 to 31 go from .300 to 9.600

		for (chan = 0; chan < 32; chan++)
		{
			if (chan >= chans)
				break;

			dest[0]	= 0x8000 + (983L * (chan + 1));
			dest[0]	|= chan << shift;	// Channel Tag
			dest++;
		}

		// Channel 32 to 63 go from -.300 to -9.600
		for (chan = 32; chan < 64; chan++)
		{
			if (chan >= chans)
				break;

			dest[0]	= 0x8000 - (983L * (chan -31));
			dest[0]	|= chan << shift;	// Channel Tag
			dest++;
		}

		dest[-1]	|= gef;				// Group End Flag

		// Channel 0 to 63 are set to zero.

		for (chan = 0; chan < chans; chan++)
		{
			dest[0]	= 0x8000;
			dest[0]	|= chan << shift;	// Channel Tag
			dest++;
		}

		dest[-1]	|= gef;				// Group End Flag
	}

	printf("Done  (2 sample square wave, per channel)\n");
	return(0);
}



//*****************************************************************************
static int _init_data(const args_t* args, int msb)
{
	int	errs;
	s32	chans	= 64;

	gsc_label ("Initialize Data");
	errs	= ao64c_query(args->fd, -1, 0, AO64C_QUERY_CHANNEL_QTY, &chans);

	if (errs)
		printf("FAIL <---\n");
	else if (args->width <= 2)
		errs	= _pattern_2(chans, msb);
	else
		errs	= _pattern_width(args, chans, msb);

	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args)
{
	os_time_ns_t	begin;
	int				errs		= 0;
	long long		data_limit	= (long long) args->tx_mb * 1024 * 1024;
	long long		ms			= 1;
	long long		ms_limit	= (long long) args->seconds * 1000;
	os_time_ns_t	now;
	double			rate;
	long			sent;
	long long		total		= 0;

	gsc_label("Writing");
	os_time_get_ns(&begin);

	for (;;)
	{
		sent	= ao64c_write(args->fd, _tx, _tx_size);

		if (sent < 0)
		{
			errs++;
			printf("FAIL <--- (ao64c_write, error %d)\n", (int) sent);
			break;
		}

		os_time_get_ns(&now);
		total	+= sent;

		if (total < data_limit)
			continue;

		ms	= ((long long) now.tv_sec * 1000 + now.tv_nsec / 1000000)
			- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);

		if (ms >= ms_limit)
			break;
	}

	if (errs == 0)
	{
		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, ");
		gsc_label_float_comma((double) ms / 1000, -1, 3);
		printf(" Seconds, ");
		rate	= (double) total * 1000 / 4 / ms;
		gsc_label_float_comma(rate, -1, 3);
		printf(" S/S)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	u32		acr;
	int		errs	= 0;
	int		msb;

	errs	+= ao64c_config_ao	(args->fd, -1, 1, OUTPUT_RATE, OUTPUT_MAP);
	errs	+= ao64c_tx_io_mode	(args->fd, -1, 1, args->io_mode, NULL);

	gsc_label ("Write Rate");
	printf("(%d second minimum, %d MB minimum)\n", args->seconds, (int) args->tx_mb);
	gsc_label_level_inc();

	errs	+= ao64c_reg_read(args->fd, -1, 0, AO64C_GSC_ACR, &acr);
	msb	= ((acr & 0xFFFF) >= 0x0400) ? 1 : 0;
	msb		= args->byte_swap ? !msb : msb;

	errs	+= _init_data(args, msb);
	errs	+= _rate_test(args);
	errs	+= ao64c_buffer_over_data	(args->fd, -1, 1, AO64C_BUFFER_OVER_DATA_CHK, NULL);
	errs	+= ao64c_buffer_over_frame	(args->fd, -1, 1, AO64C_BUFFER_OVER_FRAME_CHK, NULL);

	gsc_label_level_dec();
	return(errs);
}



