// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_auto_cal_sts.c $
// $Rev: 53036 $
// $Date: 2023-05-18 15:52:40 -0500 (Thu, 18 May 2023) $

// 16AO64C: Utilities: source file

#include "main.h"

// This module is retained for backwards compatibility.



/******************************************************************************
*
*	Function:	ao64c_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the AO64C_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao64c_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;
	s32			set		= -1;

	if (verbose)
		gsc_label_index("Autocal Status", index);

	ret		= ao64c_ioctl(fd, AO64C_IOCTL_AUTO_CAL_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AO64C_AUTO_CAL_STS_ACTIVE:

			ptr	= "Active";
			break;

		case AO64C_AUTO_CAL_STS_FAIL:

			ptr	= "Fail";
			break;

		case AO64C_AUTO_CAL_STS_PASS:

			ptr	= "Pass";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


