// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_clock_src.c $
// $Rev: 51084 $
// $Date: 2022-05-31 10:39:03 -0500 (Tue, 31 May 2022) $

// 16AO64C: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao64c_clock_src
*
*	Purpose:
*
*		Provide a visual wrapper for the AO64C_IOCTL_CLOCK_SRC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao64c_clock_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Clock Sourse", index);

	ret		= ao64c_ioctl(fd, AO64C_IOCTL_CLOCK_SRC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AO64C_CLOCK_SRC_RAG:

			ptr	= "Rate-A Generator";
			break;

		case AO64C_CLOCK_SRC_EXT:

			ptr	= "External";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


