// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_config_ao.c $
// $Rev: 53036 $
// $Date: 2023-05-18 15:52:40 -0500 (Thu, 18 May 2023) $

// 16AO64C: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_200K			(200L * 1024)



/******************************************************************************
*
*	Function:	ao64c_config_ao
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate.
*
*		map		The bitmap of the channels to be used.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao64c_config_ao(int fd, int index, int verbose, s32 fsamp, u64 map)
{
	s32	disc	= -1;
	int	errs	= 0;
	s32	nrate;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao64c_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= ao64c_tx_io_byte_swap	(fd, index, verbose, AO64C_TX_IO_BYTE_SWAP_PERFORM,	NULL);
	errs	+= ao64c_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= ao64c_tx_io_timeout		(fd, index, verbose, 30,							NULL);
	errs	+= ao64c_tx_io_over_data	(fd, index, verbose, AO64C_TX_IO_OVER_DATA_CHECK,	NULL);
	errs	+= ao64c_tx_io_over_frame	(fd, index, verbose, AO64C_TX_IO_OVER_FRAME_CHECK,	NULL);

	errs	+= ao64c_burst_enable		(fd, index, verbose, AO64C_BURST_ENABLE_NO,			NULL);
	errs	+= ao64c_rbg_enable			(fd, index, verbose, AO64C_RAG_ENABLE_NO,			NULL);

	errs	+= ao64c_buffer_mode		(fd, index, verbose, AO64C_BUFFER_MODE_OPEN,		NULL);
	errs	+= ao64c_buffer_thresh_lvl	(fd, index, verbose, _200K,							NULL);
	errs	+= ao64c_data_format		(fd, index, verbose, AO64C_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= ao64c_ground_sense		(fd, index, verbose, AO64C_GROUND_SENSE_DISABLE,	NULL);
	errs	+= ao64c_range				(fd, index, verbose, AO64C_RANGE_10V,				NULL);
	errs	+= ao64c_clock_mode			(fd, index, verbose, AO64C_CLOCK_MODE_INITIATOR,	NULL);
	errs	+= ao64c_fsamp_ao_compute	(fd, index, verbose, fsamp, &nrate,					NULL);
	errs	+= ao64c_clock_src			(fd, index, verbose, AO64C_CLOCK_SRC_RAG,			NULL);
	errs	+= ao64c_rag_nrate			(fd, index, verbose, nrate,							NULL);
	errs	+= ao64c_rag_enable			(fd, index, verbose, AO64C_RAG_ENABLE_YES,			NULL);
	errs	+= ao64c_clock_enable		(fd, index, verbose, AO64C_CLOCK_ENABLE_YES,		NULL);

	errs	+= ao64c_fsamp_ao_report_all(fd, index, verbose, map,							NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the device user manual for clarification.

	errs	+= ao64c_query				(fd, index, verbose, AO64C_QUERY_DISCONNECT, &disc);

	if (disc)
		errs	+= ao64c_output_disconnect(fd, index, verbose, AO64C_OUTPUT_DISCONNECT_YES,	NULL);

	errs	+= ao64c_autocal			(fd, index, verbose);

	if (disc)
		errs	+= ao64c_output_disconnect(fd, index, verbose, AO64C_OUTPUT_DISCONNECT_NO,	NULL);

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= ao64c_irq_sel			(fd, index, verbose, AO64C_IRQ_INIT_DONE,			NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the data overflow and frame overflow status bits.
	errs	+= ao64c_buffer_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


