// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_id.c $
// $Rev: 51375 $
// $Date: 2022-07-13 13:02:11 -0500 (Wed, 13 Jul 2022) $

// 16AO64C: Utilities: source file

#include "main.h"



//*****************************************************************************
static int _id_device_pci(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Vendor ID");
	errs	= ao64c_reg_read(fd, -1, 0, GSC_PCI_9056_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= ao64c_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9056)
	{
		printf("(PCI9056)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= ao64c_reg_read(fd, -1, 0, GSC_PCI_9056_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= ao64c_reg_read(fd, -1, 0, GSC_PCI_9056_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x3555)
	{
		printf("(16AO64C)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_acr(int fd)
{
	u32	acr;
	int	errs;

	gsc_label("Assembly Con. Reg.");

	errs	= ao64c_reg_read(fd, -1, 0, AO64C_GSC_ACR, &acr);

	if (errs == 0)
		printf("0x%08lX\n", (long) acr);

	return(errs);
}



/******************************************************************************
*
*	Function:	ao64c_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ao64c_id_device(int fd, int index, int verbose)
{
	s32			device_type;
	int			errs;
	const char*	psz		= "";

	if (verbose)
		gsc_label_index("Device", index);

	errs	= ao64c_query(fd, -1, 0, AO64C_QUERY_DEVICE_TYPE, &device_type);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (query error)\n");
	}
	else
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_16AO64C:	psz	= "16AO64C";	break;
		}

		if ((verbose) && (errs == 0))
			printf("%s\n", psz);

		gsc_label_level_inc();

		errs	+= _id_device_pci(fd);
		errs	+= _id_acr(fd);

		gsc_label_level_dec();
	}

	return(errs);
}


