// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_irq_sel.c $
// $Rev: 53036 $
// $Date: 2023-05-18 15:52:40 -0500 (Thu, 18 May 2023) $

// 16AO64C: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao64c_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AO64C_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao64c_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= ao64c_ioctl(fd, AO64C_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AO64C_IRQ_INIT_DONE:

			ptr	= "Init Done";
			break;

		case AO64C_IRQ_AUTOCAL_DONE:

			ptr	= "Autocal Done";
			break;

		case AO64C_IRQ_BUF_EMPTY:

			ptr	= "Buffer Empty";
			break;

		case AO64C_IRQ_BUF_THRESH_LO2HI:

			ptr	= "Buffer Threshold Low to High";
			break;

		case AO64C_IRQ_BUF_THRESH_HI2LO:

			ptr	= "Buffer Threshold Hi to Low";
			break;

		case AO64C_IRQ_BURST_TRIG_READY:

			ptr	= "Burst Trigger Ready";
			break;

		case AO64C_IRQ_LOAD_READY:

			ptr	= "Load Ready";
			break;

		case AO64C_IRQ_LOAD_READY_END:

			ptr	= "Load Not Ready";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


