// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_query.c $
// $Rev: 53036 $
// $Date: 2023-05-18 15:52:40 -0500 (Thu, 18 May 2023) $

// 16AO64C: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao64c_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO64C_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao64c_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AO64C_QUERY_AUTOCAL_MS:	ptr	= "Autocal Period";			break;
		case AO64C_QUERY_CHANNEL_MAX:	ptr	= "Max Channels";			break;
		case AO64C_QUERY_CHANNEL_QTY:	ptr	= "Output Channels";		break;
		case AO64C_QUERY_COUNT:			ptr	= "Query Options";			break;
		case AO64C_QUERY_DIFFERENTIAL:	ptr	= "Differential Outputs";	break;
		case AO64C_QUERY_DISCONNECT:	ptr	= "Disconnect Feature";		break;
		case AO64C_QUERY_DEVICE_TYPE:	ptr	= "Device Type";			break;
		case AO64C_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";				break;
		case AO64C_QUERY_FILTER:		ptr	= "Filter";					break;
		case AO64C_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";			break;
		case AO64C_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";				break;
		case AO64C_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";				break;
		case AO64C_QUERY_INIT_MS:		ptr	= "Initialize Period";		break;
		case AO64C_QUERY_NRATE_MASK:	ptr = "Nrate Mask";				break;
		case AO64C_QUERY_NRATE_MAX:		ptr = "Nrate Max";				break;
		case AO64C_QUERY_NRATE_MIN:		ptr = "Nrate Min";				break;
		case AO64C_QUERY_RANGE:			ptr = "Voltage Range";			break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ao64c_ioctl(fd, AO64C_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AO64C_QUERY_CHANNEL_MAX:
		case AO64C_QUERY_CHANNEL_QTY:
		case AO64C_QUERY_COUNT:
		case AO64C_QUERY_NRATE_MAX:
		case AO64C_QUERY_NRATE_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AO64C_QUERY_NRATE_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case AO64C_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AO64C_QUERY_AUTOCAL_MS:
		case AO64C_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AO64C_QUERY_FSAMP_MAX:
		case AO64C_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AO64C_QUERY_DIFFERENTIAL:
		case AO64C_QUERY_DISCONNECT:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case AO64C_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AO64C_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AO64C)
			{
				strcpy(buf, "16AO64C");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AO64C_QUERY_FILTER:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AO64C_FILTER_100KHZ:	strcpy(buf, "100KHz");	break;
				case AO64C_FILTER_10KHZ:	strcpy(buf, "10KHz");	break;
				case AO64C_FILTER_NONE:		strcpy(buf, "None");	break;
			}

			break;

		case AO64C_QUERY_RANGE:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AO64C_RANGE_BI_10_5:	strcpy(buf, "+-10V, +-5V");	break;
				case AO64C_RANGE_UNI_10_5:	strcpy(buf, "0-10V, 0-5V");	break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


