// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO64C/16AO64C_Linux_1.x.x.x.GSC_DN/utils/util_reg.c $
// $Rev: 53036 $
// $Date: 2023-05-18 15:52:40 -0500 (Thu, 18 May 2023) $

// 16AO64C: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	LABEL_WIDTH				18
#define	NAME_WIDTH				21

#define	SIZEOF_ARRAY(a)		(sizeof((a)) / sizeof((a)[0]))

#define	_GSC_REG(a)				"GSC " #a, AO64C_GSC_##a
#define	_SHOW(b,e,eol,lst,desc)	gsc_reg_field_show(LABEL_WIDTH, NAME_WIDTH, value,(b),(e),(eol),(lst),(desc))



//*****************************************************************************
static int _decode_bcr(int fd, int supported, u32 value, int width)
{
	static const char*	irq[]	=
	{
		"Initialization Completed",
		"Autocal Done",
		"Buffer Empty",
		"Buffer Threshold Status Low-to-High",
		"Buffer Threshold Status High-to-Low",
		"Burst Trigger Ready",
		"Load Ready (low-to-high)",
		"End Load Ready (high-to-low)"
	};

	static const char*	autocal[]		= { "Done",					"In Progress"			};
	static const char*	acal_status[]	= { "Fail",					"Pass"					};
	static const char*	disable[]		= { "Disable",				"Enable"				};
	static const char*	disconnect[]	= { "No (Connect)",			"Yes (Disconnect)"		};
	static const char*	initialize[]	= { "Idle",					"Initializing"			};
	static const char*	idle[]			= { "Idle",					"Active"				};
	static const char*	initiator[]		= { "Target",				"Initiator"				};
	static const char*	irq_flag[]		= { "Inactive",				"Active"				};
	static const char*	offset[]		= { "No (Twos Compliment)",	"Yes (Offset Binary)"	};
	static const char*	range[]			= { "+-10V or 0 - 10V",		"+-5V or 0 - 5V"		};
	static const char*	remote[]		= { "Disable",				"Enable (Remote)"		};

	_SHOW( 0,  0, 1, initiator,		"Clock Initiator"		);
	_SHOW( 1,  1, 1, initiator,		"Trigger Initiator"		);
	_SHOW( 2,  2, 1, disconnect,	"Disconnect Outputs"	);
	_SHOW( 3,  3, 1, remote,		"Remote Ground Sense"	);
	_SHOW( 4,  4, 1, offset,		"Offset Binary"			);
	_SHOW( 5,  5, 1, disable,		"Enable Rate-A"			);
	_SHOW( 6,  6, 1, disable,		"Enable Rate-B"			);
	_SHOW( 7,  7, 1, NULL,			"Reserved"				);
	_SHOW(10,  8, 1, irq,			"Interrupt"				);
	_SHOW(11, 11, 1, irq_flag,		"Interrupt Req. Flag"	);
	_SHOW(12, 12, 1, idle,			"Buffer Thresh. Flag"	);
	_SHOW(13, 13, 1, autocal,		"Autocalibration"		);
	_SHOW(14, 14, 1, acal_status,	"Autocal Pass Flag"		);
	_SHOW(15, 15, 1, initialize,	"Initialize"			);
	_SHOW(16, 16, 1, range,			"Output Range"			);
	_SHOW(31, 17, 1, NULL,			"Reserved"				);

	return(0);
}



//*****************************************************************************
static int _decode_bor(int fd, int supported, u32 value, int width)
{
	static const char*	b_rdy[]	= { "No",					"Yes (Will Accept)"		};
	static const char*	c_rdy[]	= { "No (Not Ready)",		"Yes (Will Accept)"		};
	static const char*	cont[]	= { "No (Continuous)",		"Yes (Burst)"			};
	static const char*	disa[]	= { "Disable",				"Enable"				};
	static const char*	empty[]	= { "No (Not Empty)",		"Yes (Empty)"			};
	static const char*	full[]	= { "No (Not Full)",		"Yes (Full)"			};
	static const char*	idle[]	= { "No (Idle)",			"Yes (Active)"			};
	static const char*	l_rdy[]	= { "No (Nor Ready)",		"Yes (Ready)"			};
	static const char*	l_req[]	= { "Idle",					"Active (Requesting)"	};
	static const char*	_open[]	= { "No (Open)",			"Yes (Circular)"		};
	static const char*	over[]	= { "No (Not Overflown)",	"Yes (Overflown)"		};
	static const char*	rag[]	= { "Rate-A Generator",		"External"				};
	static const char*	rbg[]	= { "Rate-B Generator",		"External"				};

	_SHOW( 0,  0, 1, rag,	"External Clock"	);
	_SHOW( 1,  1, 1, rbg,	"External Trigger"	);
	_SHOW( 2,  2, 1, cont,	"Burst Enable"		);
	_SHOW( 3,  3, 1, b_rdy,	"Burst Ready"		);
	_SHOW( 4,  4, 1, idle,	"SW Burst Trigger"	);
	_SHOW( 5,  5, 1, disa,	"Enable Clocking"	);
	_SHOW( 6,  6, 1, c_rdy,	"Clock Ready"		);
	_SHOW( 7,  7, 1, idle,	"SW Clock"			);
	_SHOW( 8,  8, 1, _open,	"Circular Buffer"	);
	_SHOW( 9,  9, 1, l_req,	"Load Request"		);
	_SHOW(10, 10, 1, l_rdy,	"Load Ready"		);
	_SHOW(11, 11, 1, idle,	"Clear Buffer"		);
	_SHOW(12, 12, 1, empty,	"Empty Buffer"		);
	_SHOW(14, 13, 1, NULL,	"Reserved"			);
	_SHOW(15, 15, 1, full,	"Buffer Full"		);
	_SHOW(16, 16, 1, over,	"Buffer Overflow"	);
	_SHOW(17, 17, 1, over,	"Frame Overflow"	);
	_SHOW(31, 18, 1, NULL,	"Reserved"			);

	return(0);
}



//*****************************************************************************
static int _decode_acr(int fd, int supported, u32 value, int width)
{
	static const char*	chans[]	=
	{
		"64 Single Ended",
		"32 Single Ended",
		"32 Differential",
		"16 Differential"
	};

	static const char*	scale[]	=
	{
		"Unipolar: 0-10V, 0-5V",
		"Bipolar: +-10V, +-5V",
		"RESERVED",
		"RESERVED"
	};

	static const char*	filtr[]	=
	{
		"100KHz",
		"10KHz",
		"No filter",
		"RESERVED"
	};

	static const char*	clock[]	=
	{
		"49.152MHz",
		"RESERVED",
		"RESERVED",
		"RESERVED"
	};

	static const char*	no[]	= { "No (Absent)",	"Yes (Present)"	};

	s32	v;

	_SHOW(15,  0, 0, NULL,	"Firmware Revision"	);
	v	= GSC_FIELD_DECODE(value, 15, 0);
	printf("%04lX\n", (long) v);

	_SHOW(17, 16, 1, chans,	"Output Channels"		);
	_SHOW(19, 18, 1, scale,	"Output Scale"			);
	_SHOW(21, 20, 1, filtr,	"Output Filter"			);
	_SHOW(23, 22, 1, clock,	"Master Clock Freq."	);
	_SHOW(24, 24, 1, no,	"Output Disconnect"		);
	_SHOW(31, 25, 1, NULL,	"Reserved"				);

	return(0);
}



//*****************************************************************************
static int _decode_odbr(int fd, int supported, u32 value, int width)
{
	_SHOW(15,  0, 1, NULL,	"Data"				);
	_SHOW(21, 16, 1, NULL,	"Channel Tag"		);
	_SHOW(22, 22, 1, NULL,	"Last Channel Tag"	);
	_SHOW(23, 23, 1, NULL,	"EOF Tag"			);
	_SHOW(31, 24, 1, NULL,	"Reserved"			);

	return(0);
}



//*****************************************************************************
static int _decode_bsr(int fd, int supported, u32 value, int width)
{
	_SHOW(18,  0, 1, NULL,	"Buffer Size"	);
	_SHOW(31, 19, 1, NULL,	"Reserved"		);

	return(0);
}



//*****************************************************************************
static int _decode_btr(int fd, int supported, u32 value, int width)
{
	static const char*	flag[]	= { "Clear (<= Threshold)",	"Set (> Threshold)"	};

	_SHOW(18,  0, 1, NULL,	"Buffer Threshold"	);
	_SHOW(19, 19, 1, NULL,	"Reserved"			);
	_SHOW(20, 20, 1, flag,	"Threshold Flag"	);
	_SHOW(31, 21, 1, NULL,	"Reserved"			);

	return(0);
}



//*****************************************************************************
static int _decode_rxgr(int fd, int supported, u32 value, int width)
{
	_SHOW(23,  0, 1, NULL,	"Nrate"		);
	_SHOW(31, 24, 1, NULL,	"Reserved"	);

	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	// name	reg			err	value	ask_support	decode			desc
	{ _GSC_REG(BCR),	0,	0,		0,			_decode_bcr,	"Board Control Register"			},
	{ _GSC_REG(BOR),	0,	0,		0,			_decode_bor,	"Buffer Operations Register"		},
	{ _GSC_REG(ACR),	0,	0,		0,			_decode_acr,	"Assembly Configuration Register"	},
	{ _GSC_REG(AVR),	0,	0,		0,			NULL,			"Autocal Values Register"			},
	{ _GSC_REG(ODBR),	0,	0,		0,			_decode_odbr,	"Output Data Buffer Register"		},
	{ _GSC_REG(BSR),	0,	0,		0,			_decode_bsr,	"Buffer Size Register"				},
	{ _GSC_REG(BTR),	0,	0,		0,			_decode_btr,	"Buffer Threshold Register"			},
	{ _GSC_REG(RAGR),	0,	0,		0,			_decode_rxgr,	"Rate-A Generator Register"			},
	{ _GSC_REG(RBGR),	0,	0,		0,			_decode_rxgr,	"Rate-B Generator Register"			},
	{ NULL }
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	ao64c_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int ao64c_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, ao64c_reg_read);
	return(errs);
}



/******************************************************************************
*
*	Function:	ao64c_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao64c_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	ao64c_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* ao64c_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	ao64c_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao64c_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	ao64c_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* ao64c_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}


